# Copyright (c) 2008-2009 gocept gmbh & co. kg
# See also LICENSE.txt

import random
import string
import crypt
import re

import grok
import zope.component
import zope.interface
import zope.schema

import gocept.directory.form
import gocept.directory.interfaces
import gocept.directory.unixaccount

SALT_CHARACTERS = string.ascii_letters + string.digits + string.punctuation


class UserAccount(gocept.directory.unixaccount.UnixAccount):

    grok.implements(gocept.directory.interfaces.IUserAccount)

    password2 = u''

    def __init__(self, username='', password=u''):
        super(UserAccount, self).__init__(username)
        self.password = password

    def get_password(self):
        return self._password

    def set_password(self, value):
        self._password = value
        if value:
            salt = ''.join(random.sample(SALT_CHARACTERS, 8))
            self.crypted_password = crypt.crypt(value, '$1$%s$' % salt)
        else:
            self.crypted_password = u'!'

    password = property(fget=get_password, fset=set_password)


class Index(gocept.directory.form.BasicEditForm):
    pass


class UserAccountOwnerCandidates(gocept.directory.interfaces.Persons):

    def getValues(self):
        owners = zope.component.getUtility(
            gocept.directory.interfaces.IPersons).sorted_values()
        return tuple(person for person in owners
                     if person.unixaccount is None)


class IUserAccountAdding(zope.interface.Interface):

    owner = zope.schema.Choice(
        title=u'Owner', source=UserAccountOwnerCandidates(),
        constraint=lambda person: (person.unixaccount is None or
                                   person.unixaccount.owner is person))
    username = zope.schema.ASCIILine(
        title=u'User login',
        min_length=2,
        max_length=8,
        constraint=re.compile(r'[a-z0-9_-]+$').match)


@grok.implementer(IUserAccountAdding)
def account_with_candidates(account):
    zope.interface.directlyProvides(account, IUserAccountAdding)
    return account
